/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  Controller.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Sat Jan 10 2004
//
//  $Id: Controller.m,v 1.14 2007/10/18 00:02:27 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// include files
/////////////////////////////////////////////////////////////////////

#import "Controller.h"
#import "SSHAgent.h"
#import "KeychainHandler.h"
#import "PrefController.h"
#import "EnvFile.h"
#include <unistd.h>

/////////////////////////////////////////////////////////////////////
// class Controller
/////////////////////////////////////////////////////////////////////

@interface Controller ( PrivateMethods )
- (void) startAgent;
- (void) stopAgent;
- (void) restartAgent;
- (BOOL) checkExplicitSocketPath:(NSString*)socket;
- (void) setupEnvFile;
- (void) dispatchKeychainError:(NSString*)errorString;
- (void) dispatchAgentError:(NSString*)errorString;
@end

@implementation Controller

- (id) init
{
   if ( self = [super init] )
   {
      m_socketPathChanged = NO;

      // NSApplication notifications tell us when to start and stop
      NSNotificationCenter* nc = [NSNotificationCenter defaultCenter];
      [nc addObserver:self
         selector:@selector( appDidLaunch: )
         name:NSApplicationDidFinishLaunchingNotification
         object:nil];
      [nc addObserver:self
         selector:@selector( appWillTerminate: )
         name:NSApplicationWillTerminateNotification
         object:nil];
      
      // PrefController notifications tell us when to restart agent
      // and or remake the environment file
      [nc addObserver:self
          selector:@selector( socketPathDidChange: )
          name:SocketPathChangedNotification
          object:nil];
      [nc addObserver:self
          selector:@selector( agentEnvFileDidChange: )
          name:AgentEnvFileChangedNotification
          object:nil];
      [nc addObserver:self
          selector:@selector( agentRestartRequest: )
          name:AgentRestartRequestNotification
          object:nil];
      [nc addObserver:self
          selector:@selector( preferencesSaved: )
          name:SavePreferencesNotification
          object:nil];
   }
   return( self );
}

- (void) dealloc
{
   NSNotificationCenter* nc = [NSNotificationCenter defaultCenter];
   [nc removeObserver:self];
   [m_agent release];
   [m_envFile release];
   [super dealloc];
}

- (void) appDidLaunch:(NSNotification*)notif
{
   [self restartAgent];
}

- (void) appWillTerminate:(NSNotification*)notif
{
   [self stopAgent];
}

- (void) socketPathDidChange:(NSNotification*)notif
{
   // when preferences are saved, force an agent restart
   m_socketPathChanged = YES;
}

- (void) agentEnvFileDidChange:(NSNotification*)notif
{
   [self setupEnvFile];
   if ( m_envFile != nil )
      [m_envFile updateForAgent:m_agent];
}

- (void) agentRestartRequest:(NSNotification*)notif
{
   [self restartAgent];
}

- (void) preferencesSaved:(NSNotification*)notif
{
   // if the socket path was changed, always restart the agent;
   // otherwise, only start it if it is not already running (i.e.
   // because an error occurred the last time)
   if ( m_socketPathChanged )
   {
      [self restartAgent];
      m_socketPathChanged = NO;
   }
   else if ( m_agent == nil )
      [self startAgent];
}

@end

@implementation Controller ( PrivateMethods )

- (void) startAgent
{
   // grab the pref controller
   PrefController* prefs = [PrefController sharedPrefController];
   NSString* socket = [prefs explicitSocketPath];

   // before starting the agent, check to make sure the explicit
   // socket path (if any) doesn't already exist, and give the 
   // user the opportunity to overwrite it if it does; this deals
   // with problems when the previous session (or system) crashed,
   // instead of just reporting that ssh-agent failed to start
   if ( ! [self checkExplicitSocketPath:socket] )
      return;

   // launch the agent using the given socket path (or nil if none,
   // which means the agent will assign a default name)
   m_agent = [[SSHAgent alloc] initWithSocketName:socket errorDelegate:self];
   if ( ! [m_agent launch] )
   {
      // error dispatched: release the agent so we know it's not running
      [m_agent shutdown]; // likely a no-op
      [m_agent release];
      m_agent = nil;
      return;
   }
   
   // look in the keychain(s) for passwords with the right service
   // name, and add any accounts found there
   KeychainHandler* kc = 
      [[KeychainHandler alloc] initWithService:@"ssh-agent"
                               errorDelegate:self];
   if ( kc != nil )
   {
      NSEnumerator* acctEnum = [kc accountEnumerator];
      NSString* acct;
      while ( acct = [acctEnum nextObject] )
      {
         NSString* pass = [kc passphraseForAccount:acct];
         [m_agent addKey:acct usingPassword:pass];
      }
      [kc release];
   }
   
   // update the env file if we have one
   if ( m_envFile != nil )
      [m_envFile updateForAgent:m_agent];
}

- (void) stopAgent
{
   if ( m_agent != nil )
   {
      [m_agent shutdown];
      [m_agent release];
      m_agent = nil;
   }
   if ( m_envFile != nil )
   {
      [m_envFile clear];
      [m_envFile release];
      m_envFile = nil;
   }
}

- (void) restartAgent
{
   [self stopAgent];
   [self setupEnvFile];
   [self startAgent];
}

- (BOOL) checkExplicitSocketPath:(NSString*)socket
{
   if ( socket == nil )
      return( YES );

   NSFileManager* fm = [NSFileManager defaultManager];
   if ( ! [fm fileExistsAtPath:socket] )
      return( YES );

   // if we are restarting the agent, we can get into an ugly
   // race condition, where the socket isn't gone yet when we
   // do the above check; so *only* if we are considering
   // issuing an error, sleep briefly and check again
   sleep( 1 );
   if ( ! [fm fileExistsAtPath:socket] )
      return( YES );

   // since we are (normally) faceless, need to force activiation
   // or the dialog will get lost and never be seen
   [NSApp activateIgnoringOtherApps:YES];
   int rc = NSRunAlertPanel( @"Problem Initializing ssh-agent",
                             @"The specified authorization socket path "
                             @"\"%@\" already exists. Unless you have "
                             @"started ssh-agent from outside of SSHChain, "
                             @"this socket is probably left over from a "
                             @"previous session and can be safely overwritten.",
                             @"Overwrite",     // default
                             @"Quit SSHChain", // alternate (leftmost)
                             @"Configure...",  // other
                             socket );
   switch ( rc )
   {
   case NSAlertDefaultReturn:   // Overwrite
      // if we cannot delete it, will catch ssh-agent error
      [fm removeFileAtPath:socket handler:nil];
      return( YES );
   case NSAlertAlternateReturn: // Quit
      [NSApp terminate:self];
      break;
   case NSAlertOtherReturn:     // Configure
      // make sure pref window is open
      [[[PrefController sharedPrefController] window] makeKeyAndOrderFront:nil];
      break;
   }

   return( NO );
}

- (void) setupEnvFile
{
   // remove any existing file
   if ( m_envFile != nil )
   {
      [m_envFile clear];
      [m_envFile release];
      m_envFile = nil;
   }

   // grab the pref controller and get path and format
   PrefController* prefs = [PrefController sharedPrefController];
   NSString* path = [prefs agentEnvironmentFile];
   PrefControllerAgentShell shell = [prefs agentEnvironmentShell];
   
   // bail if we don't want an env file
   if ( path == nil )
      return;
   
   // otherwise make the file
   m_envFile = [[EnvFile alloc] initWithPath:path
                                cshFormat:( shell == AGENT_SHELL_CSH )];
   
   // file will be actually create when agent is updated
}

- (void) dispatchAgentError:(NSString*)errorString
{
   // since we are (normally) faceless, need to force activiation
   // or the dialog will get lost and never be seen
   [NSApp activateIgnoringOtherApps:YES];
   int rc = NSRunAlertPanel( @"Problem Initializing ssh-agent",
                             errorString,
                             @"Relaunch Agent", // default
                             @"Quit SSHChain",  // alternate (leftmost)
                             @"Configure..."    // other
                           );

   switch ( rc )
   {
   case NSAlertDefaultReturn:   // Relaunch
      [self restartAgent];
      break;
   case NSAlertAlternateReturn: // Quit
      [NSApp terminate:self];
      break;
   case NSAlertOtherReturn:     // Configure
      // make sure pref window is open
      [[[PrefController sharedPrefController] window] makeKeyAndOrderFront:nil];
      break;
   }
}

- (void) dispatchKeychainError:(NSString*)errorString
{
   // since we are (normally) faceless, need to force activiation
   // or the dialog will get lost and never be seen
   [NSApp activateIgnoringOtherApps:YES];
   int rc = NSRunAlertPanel( @"Problem Accessing Keychain",
                             errorString,
                             @"Relaunch Agent", // default
                             @"Quit SSHChain",  // alternate (leftmost)
                             @"Ignore"          // other
                           );
   switch ( rc )
   {
   case NSAlertDefaultReturn:   // Relaunch
      [self restartAgent];
      break;
   case NSAlertAlternateReturn: // Quit
      [NSApp terminate:self];
      break;
   case NSAlertOtherReturn:     // Ignore
      // ignore the inaccessible key (there may be others)
      break;
   }
}

@end
