/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  KeychainHandler.h
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Sat Jan 10 2004
//
//  $Id: KeychainHandler.h,v 1.8 2004/04/04 00:16:21 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

/////////////////////////////////////////////////////////////////////
// class KeychainHandler
/////////////////////////////////////////////////////////////////////

/*!
 * @class KeychainHandler
 *
 * The KeychainHandler class provides read-only access into the
 * user Keychain for the SSHChain application (wrapping the
 * C-based SecKeychain API in an Objective-C interface).
 * A KeychainHandler object is created for a given service name,
 * and can be queried for all of the matching account names and
 * their passwords as found in the default keychain.
 */
@interface KeychainHandler : NSObject
{
   NSString*     m_service;
   NSDictionary* m_data;
   id            m_delegate;
}

/*!
 * @method initWithService:
 *
 * Create a keychain handler for the named service. The resulting
 * handler can be used to query all accounts, and their passphrases,
 * for the named service.
 *
 * If there is an error searching the keychain(s), nil is returned.
 * Error messages will be dispatched through the delegate, if any.
 * See the KeychainHandler_ErrorDelegate category for the informal
 * protocol to be implemented by the delegate object.
 *
 * Merely finding no matches is not considered an error and will
 * not return nil; check accountCount for the number of matches.
 *
 * @param service
 *    name of the service to query
 * @param delegate
 *    delegate for dispatching errors, or nil if none
 * @result
 *    initialized KeychainHandler object, or nil if error
 */
- (id) initWithService:(NSString*)service
       errorDelegate:(id)delegate;

/*!
 * @method accountCount
 *
 * @result
 *    number of accounts for service; may be zero
 */
- (int) accountCount;

/*!
 * @method accountEnumerator
 *
 * Get an enumerator over all account names for service.
 *
 * @result
 *    enumerator over accounts as strings
 */
- (NSEnumerator*) accountEnumerator;

/*!
 * @method passphraseForAccount:
 *
 * Retrieve the passphrase for the named account. If the account
 * name is one from accountEnumerator, it should always return
 * a non-nil passphrase string.
 *
 * @param account
 *    the account name to get passphrase for
 * @result
 *    the passphrase, or nil if account not found
 */
- (NSString*) passphraseForAccount:(NSString*)account;

/*!
 * @method passphraseForService:withAccount:
 *
 * A convenience method for finding a single passphrase, knowing
 * both the service and account name.
 *
 * @param service
 *    the service to get passphrase for
 * @param account
 *    the account name to get passphrase for
 * @result
 *    the passphrase, or nil if not found
 */
+ (NSString*) passphraseForService:(NSString*)service
              withAccount:(NSString*)account;

@end

/////////////////////////////////////////////////////////////////////
#pragma mark ERROR DELEGATE INFORMAL PROTOCOL
/////////////////////////////////////////////////////////////////////

/*!
 * @category NSObject ( KeychainHandler_ErrorDelegate )
 *
 * Informal protocol used for KeychainHandler's error delegate.
 */
@interface NSObject ( KeychainHandler_ErrorDelegate )

/*!
 * @method dispatchKeychainError:
 *
 * Error messages are dispatched through this delegate method.
 *
 * @param errorString
 *    a user-appropriate string describing the error encountered
 */   
- (void) dispatchKeychainError:(NSString*)errorString;

@end

